#/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Compat is a series of modules compatible with hardware brands other
than Xtralien.
The goal is to make creating any experiment as easy as using an
Xtralien Device.
"""
